@@ -186,7 +186,7 @@ class CeleryParaGlider(ParaGlider):
 
     def consume(
         self,
-        data,
+        data=None,
         cleanup=None,
         split_count=None,
         synchronous=False,
@@ -197,7 +197,7 @@ class CeleryParaGlider(ParaGlider):
 
         Parameters
         ----------
-        data
+        data : iterable, optional
             Iterable of data to consume
         cleanup : dict, optional
             A mapping of arg names to clean up functions to be run after
@@ -220,7 +220,11 @@ class CeleryParaGlider(ParaGlider):
             split_count = len(app_stats.keys())
 
         split_count = split_count_helper(data, split_count)
-        splits = divide_data(data, split_count)
+        if data is None:
+            splits = [None for s in range(split_count)]
+        else:
+            splits = divide_data(data, split_count)
+
         dbg(
             "%s: data len: %s, splits: %d"
             % (self.__class__.__name__, size(data, "n/a"), split_count)
