@@ -2,7 +2,9 @@
 
 import argparse
 import asyncio
+from collections import defaultdict
 import configparser
+from contextlib import contextmanager
 import datetime
 from dateutil import parser as dateparser
 from dateutil.relativedelta import relativedelta
@@ -25,6 +27,8 @@ from pyexcel_xlsx import get_data as get_xlsx, save_data as save_xlsx
 from pyexcel_xls import get_data as get_xls, save_data as save_xls
 from tlbx import (
     st,
+    raiseif,
+    raiseifnot,
     json,
     dbg as _dbg,
     dbgsql as _dbgsql,
@@ -119,7 +123,7 @@ def to_datetime(d):
         return datetime.datetime.combine(d, datetime.time()).replace(microsecond=0)
     if type(d) == str:
         return to_datetime(date_from_str(d))
-    assert False, "Unsupported type: %s" % type(d)
+    raise AssertionError("Unsupported type: %s" % type(d))
 
 
 def to_date(d):
@@ -129,7 +133,7 @@ def to_date(d):
         return d.date()
     if type(d) == str:
         return to_date(date_from_str(d))
-    assert False, "Unsupported type: %s" % type(d)
+    raise AssertionError("Unsupported type: %s" % type(d))
 
 
 def datetime_cmp(d1, d2):
@@ -200,12 +204,14 @@ def get_datetime_windows(
         A list of tuples of start / end datetime pairs
 
     """
-    assert (
-        window_size_hours or num_windows
-    ), "One of window_size_hours or num_windows must be specified"
-    assert not (
-        window_size_hours and num_windows
-    ), "Only one of window_size_hours or num_windows can be specified"
+    raiseifnot(
+        window_size_hours or num_windows,
+        "One of window_size_hours or num_windows must be specified",
+    )
+    raiseif(
+        window_size_hours and num_windows,
+        "Only one of window_size_hours or num_windows can be specified",
+    )
 
     start_date = to_datetime(start_date)
     end_date = to_datetime(end_date)
@@ -256,9 +262,9 @@ def _process_date_action_dest(dest, values, end_date):
             start_date = date_from_str(values[0])
             end_date = date_from_str(values[1])
         else:
-            assert False, "date_range only accepts one or two arguments"
+            raise AssertionError("date_range only accepts one or two arguments")
     else:
-        assert False, "Unrecognized date action %s" % dest
+        raise AssertionError("Unrecognized date action %s" % dest)
     return start_date, end_date
 
 
@@ -315,12 +321,14 @@ class DateWindowAction(argparse.Action):
         end_date = today + datetime.timedelta(days=1)
         start_date, end_date = _process_date_action_dest(self.dest, values, end_date)
 
-        assert isinstance(
-            start_date, datetime.date
-        ), "start date must be a datetime.date object"
-        assert isinstance(
-            end_date, datetime.date
-        ), "end date must be a datetime.date object"
+        raiseifnot(
+            isinstance(start_date, datetime.date),
+            "start date must be a datetime.date object",
+        )
+        raiseifnot(
+            isinstance(end_date, datetime.date),
+            "end date must be a datetime.date object",
+        )
 
         date_windows = get_date_windows(
             start_date, end_date, reverse=getattr(namespace, "reverse_windows", False)
@@ -459,10 +467,10 @@ def divide_data(data, n):
 
 
 def flatten(l):
-    """Flatten a list of lists"""
-    assert (
-        l and len(l) > 0
-    ), "flatten requires a list of lists or list of pandas objects"
+    """Flatten a list of iterables"""
+    raiseifnot(
+        l and len(l) > 0, "flatten requires a list of lists or list of pandas objects"
+    )
     if is_pandas(l[0]):
         return pd.concat(l)
     else:
@@ -512,10 +520,54 @@ def listify(o):
     return [o]
 
 
+def join(tables, on=None, how="left", rsuffixes=None):
+    """Join a list of iterables or DataFrames"""
+
+    if rsuffixes:
+        raiseifnot(
+            len(rsuffixes) == (len(tables) - 1), "Must have one rsuffix per right table"
+        )
+
+    convert_df = True
+    if isinstance(tables[0], pd.DataFrame):
+        raiseifnot(
+            all([isinstance(t, pd.DataFrame) for t in tables]),
+            "Expected all items to be DataFrames",
+        )
+        convert_df = False
+
+    df = None
+    for i, table in enumerate(tables):
+        if convert_df:
+            table = pd.DataFrame.from_records(table, index=on, coerce_float=True)
+            # https://pandas.pydata.org/pandas-docs/stable/user_guide/missing_data.html
+            # Force new dtypes to avoid int->float coersion when null values are present
+            table = table.convert_dtypes()
+
+        if i == 0:
+            df = table
+            continue
+
+        rsuffix = ""
+        if rsuffixes:
+            rsuffix = rsuffixes[i - 1]
+
+        df = df.join(table, on=on, how=how, rsuffix=rsuffix)
+
+    if not convert_df:
+        return df
+
+    if how != "outer":
+        df = df.reset_index()
+    df.replace({np.nan: None}, inplace=True)
+    rows = df.to_dict(orient="records")
+    return rows
+
+
 # -------- File utils
 
 
-def open_filepath_or_buffer(f, open_flags="r", compression=None):
+def open_filepath_or_buffer(f, open_flags="r", compression=None, is_text=True):
     """Use pandas IO functions to return a handle from a filepath
     or buffer.
 
@@ -527,6 +579,8 @@ def open_filepath_or_buffer(f, open_flags="r", compression=None):
         mode to open file
     compression : str, optional
         compression arg passed to pandas functions
+    is_text : bool
+        Whether file/buffer is in text format, Passed through to pandas helpers.
 
     Returns
     -------
@@ -544,11 +598,25 @@ def open_filepath_or_buffer(f, open_flags="r", compression=None):
     if isinstance(f, str):
         close = True
 
-    f, handles = get_handle(f, open_flags, compression=compression)
+    f, handles = get_handle(f, open_flags, compression=compression, is_text=is_text)
 
+    # TODO: review behavior when handles has multiple files
     return f, handles, close
 
 
+@contextmanager
+def get_file_handle(*args, **kwargs):
+    """Context manager pass through to open_filepath_or_buffer. This will
+    automatically close the file if and only if it was opened here. If file
+    handles are passed in it is assumed the caller will manage them."""
+    f, _, close = open_filepath_or_buffer(*args, **kwargs)
+    try:
+        yield f
+    finally:
+        if close:
+            f.close()
+
+
 # -------- Excel utils
 
 
@@ -559,7 +627,7 @@ def excel_file_type(f):
             return XLS
         if f.endswith(XLSX):
             return XLSX
-        assert False, "Unsupported Excel file: %s" % f
+        raise AssertionError("Unsupported Excel file: %s" % f)
     else:
         if hasattr(f, "name") and f.name.endswith(XLS):
             return XLS
